<?php

App::uses('AppHelper', 'View/Helper');

class ImgHelper extends AppHelper {

	public $helpers = array(
		'Html'
	);

	public function __construct(View $View, $settings = []) {
		if ($settings == null) {
			$settings = array();
		}

		parent::__construct($View, $settings);

		$defaults = array(
			'uploadDirectory' => 'files' . DS . 'image',
			'cacheDirectory' => 'files' . DS . 'img_cache',
			'memoryLimit' => '1024M'
		);

		$this->settings = array_merge($defaults, $settings);

		return;
	}

	/**
	 * Resizes an image.
	 *
	 * @param array|string Image to resize
	 * @param array $dimensions Dimensions for resized image
	 * @param array $options Img tag attributes
	 * @param string $context Image context (defines a directory to place image)
	 * @return string fully qualified img tag
	 */
	public function resize($image, $dimensions = null, $options = [], $context = null) {
		$path = $this->_path($image, $dimensions, $context);

		if (!empty($image['alt']) && empty($options['alt'])) {
			$options['alt'] = $image['alt'];
		} elseif (empty($options['alt']) && !empty($image['name'])) {
			$options['alt'] = $image['name'];
		}

		// Make sure we apply the dimension attributes to the output image to
		// improve browser performance (this allows the browser to know how
		// much space to allocate the image before it fully renders).
		if (empty($options['width']) && !empty($dimensions['width'])) {
			$options['width'] = $dimensions['width'];
		}
		if (empty($options['height']) && !empty($dimensions['height'])) {
			$options['height'] = $dimensions['height'];
		}

		return $this->Html->image($path, $options);
	}

	/**
	 * Wrapper method so that we can directly get a path to a cached resized image. This is to get
	 * round issues with the second parameter needing to be passed by reference.
	 *
	 * @param array|string Image to resize
	 * @param array $dimensions Dimensions for resized image
	 * @param string $context Image context (defines a directory to place image)
	 * @return string Path to resized image
	 */
	public function path($image, $dimensions = null, $context = null) {
		return $this->_path($image, $dimensions, $context);
	}

	/**
	 * Returns the path to a cached resized image.
	 *
	 * @param array|string Image to resize
	 * @param array $dimensions Dimensions for resized image
	 * @param string $context Image context (defines a directory to place image)
	 * @return string Path to resized image
	 */
	protected function _path($image, &$dimensions = null, $context = null) {
		$isPdf = !empty($dimensions['pdf']);

		$cacheFilepath = $this->settings['cacheDirectory'] . DS;

		if (! empty($image)) {
			if (is_array($image)) {

				$originalFilename = $image['filename'];
				$originalFilepath = $this->settings['uploadDirectory'] . DS . $image['id'] . DS;

				$cacheFilepath .= ($context !== null ? $context : $image['id']) . DS;

				$isPdf = in_array($image['type'], ['application/pdf', 'application/postscript']);

			} else {

				$parts = pathinfo($image);

				$originalFilename = $parts['basename'];
				$originalFilepath = DS . $parts['dirname'] . DS;

				$cacheFilepath .= $context !== null ? $context . DS : '';

			}
		}

		if (
			empty($image) ||
			! file_exists(WWW_ROOT . $originalFilepath . $originalFilename)
		) {
			// Setup no image
			$originalFilepath = 'img' . DS;
			$originalFilename = 'no-image.png';
		}

		// Create a filename for the cached image.
		$cacheOptions = array_intersect_key(
			$dimensions,
			array_flip(array('width', 'height', 'crop', 'fit'))
		);
		$cacheFilename = $cacheFilepath . implode('_', $cacheOptions) . '_' . $originalFilename;
		$cacheFilename = $isPdf === true ? preg_replace('|(\.[a-z0-9]+)$|i', '.png', $cacheFilename) : $cacheFilename;

		if (!file_exists(WWW_ROOT . $cacheFilename)) {

			$Image = new imagick();

			if ($isPdf === true) {
				$Image->setResolution(300, 300);
				$Image->setBackgroundColor('#ffffff');
				$Image->readImage(WWW_ROOT . $originalFilepath . $originalFilename . '[0]');
				$Image = $Image->mergeImageLayers(Imagick::LAYERMETHOD_FLATTEN);
				$Image->setImageFormat('png');
			} else {
				try {
					$Image->readImage(WWW_ROOT . $originalFilepath . $originalFilename);
				} catch (ImagickException $e) {
					CakeLog::write('error', 'EvImg: Invalid Image Format for file ' . $originalFilepath . $originalFilename);

					// Show placeholder image instead.
					$Image->readImage(WWW_ROOT . DS . 'img' . DS . 'no-image.png');
				}
			}

			$height = $Image->getImageHeight();
			$width = $Image->getImageWidth();

			if (!empty($dimensions['background'])) {
				$Image->setbackgroundcolor($dimensions['background']);
				$Image->setimagebackgroundcolor($dimensions['background']);
			}

			if (!empty($dimensions['crop'])) {

				$Image->cropThumbnailImage($dimensions['width'], $dimensions['height']);

			} elseif (empty($dimensions['width'])) {

				if ((int)$dimensions['height'] < $height) {
					$Image->thumbnailImage(0, (int)$dimensions['height']);
				}

			} elseif (empty($dimensions['height'])) {

				if ((int)$dimensions['width'] < $width) {
					$Image->thumbnailImage((int)$dimensions['width'], 0);
				}

			} elseif (!empty($dimensions['fit'])) {

				$Image->thumbnailImage($dimensions['width'], $dimensions['height'], true, true);

			} elseif ($height <= $width) {

				if ((int)$dimensions['width'] < $width) {
					$Image->thumbnailImage((int)$dimensions['width'], 0);
				}

			} else {

				if ((int)$dimensions['height'] < $height) {
					$Image->thumbnailImage(0, (int)$dimensions['height']);
				}

			}

			if (!file_exists(WWW_ROOT . $cacheFilepath)) {
				// cache directory doesn't exist - create it now
				mkdir(WWW_ROOT . $cacheFilepath, 0777, true);
			}

			$dimensions = $Image->getImageGeometry();

			$Image->setImageCompressionQuality(0.75);

			// Strip image of all profiles and comments to reduce size.
			$Image->stripImage();

			$Image->writeImage(WWW_ROOT . $cacheFilename);

		} else {
			// Correct image dimensions to be used on the <img/> tag.
			$Image = new imagick();
			$Image->readImage(WWW_ROOT . $cacheFilename);
			$dimensions = $Image->getImageGeometry();
		}

		// Return the image path (make sure we use forward and not back slashes
		// for the image, this is an issue when using a Windows server).
		return '/' . str_replace('\\', '/', $cacheFilename);
	}

}
